# ALMA Data Reduction Script

# Calibration

thesteps = []
step_title = {0: 'listobs',
              1: 'A priori flagging',
              2: 'Generation and time averaging of the WVR cal table',
              3: 'Generation of the Tsys cal table',
              4: 'Application of the WVR and Tsys cal tables',
              5: 'Split out science SPWs and time average',
              6: 'Listobs, clear pointing table, and save original flags',
              7: 'Initial flagging',
              8: 'Putting a model for the flux calibrator(s)',
              9: 'Save flags before bandpass cal',
              10: 'Bandpass calibration',
              11: 'Save flags before gain cal',
              12: 'Gain calibration',
              13: 'Save flags before applycal cal',
              14: 'Application of the bandpass and gain cal tables'}

try:
  print 'List of steps to be executed ...', mysteps
  thesteps = mysteps
except:
  print 'global variable mysteps not set.'
if (thesteps==[]):
  thesteps = range(0,len(step_title))
  print 'Executing all steps: ', thesteps

# The Python variable 'mysteps' will control which steps
# are executed when you start the script using
#   execfile('scriptForCalibration.py')
# e.g. setting
#   mysteps = [2,3,4]# before starting the script will make the script execute
# only steps 2, 3, and 4
# Setting mysteps = [] will make it execute all steps.

import re

# es = au.stuffForScienceDataReduction() 


if re.search('^3.4', casadef.casa_version) == None:
 sys.exit('ERROR: PLEASE USE THE SAME VERSION OF CASA THAT YOU USED FOR GENERATING THE SCRIPT: 3.4')


# CALIBRATE_AMPLI: Titan
# CALIBRATE_ATMOSPHERE: 3c279,HD 100546,J1147-6753s4,Titan
# CALIBRATE_BANDPASS: 3c279
# CALIBRATE_FOCUS: 
# CALIBRATE_PHASE: 3c279,J1147-6753s4,Titan
# CALIBRATE_POINTING: 3c279,J1147-6753s4,J1337-129
# OBSERVE_TARGET: HD 100546

# Using reference antenna = DV04

print "# A priori calibration"

# listobs
mystep = 0
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_X552607_Xa39.ms.listobs')
  listobs(vis = 'uid___A002_X552607_Xa39.ms',
    listfile = 'uid___A002_X552607_Xa39.ms.listobs')
  
  

# A priori flagging
mystep = 1
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  tflagdata(vis = 'uid___A002_X552607_Xa39.ms',
    mode = 'manual',
    autocorr = T,
    flagbackup = F)
  
  tflagdata(vis = 'uid___A002_X552607_Xa39.ms',
    mode = 'manual',
    intent = '*POINTING*,*SIDEBAND_RATIO*,*ATMOSPHERE*',
    flagbackup = F)
  
  flagcmd(vis = 'uid___A002_X552607_Xa39.ms',
    inpmode = 'table',
    action = 'plot')
  
  flagcmd(vis = 'uid___A002_X552607_Xa39.ms',
    inpmode = 'table',
    action = 'apply')
  

# Generation and time averaging of the WVR cal table
mystep = 2
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_X552607_Xa39.ms.wvr') 
  
  mylogfile = casalog.logfile()
  casalog.setlogfile('uid___A002_X552607_Xa39.ms.wvrgcal')
  
  wvrgcal(vis = 'uid___A002_X552607_Xa39.ms',
    caltable = 'uid___A002_X552607_Xa39.ms.wvr',
    tie = ['HD 100546,J1147-6753s4'],
    statsource = 'HD 100546')
  
  casalog.setlogfile(mylogfile)
  
  os.system('rm -rf uid___A002_X552607_Xa39.ms.wvr.smooth') 
  
  smoothcal(vis = 'uid___A002_X552607_Xa39.ms',
    tablein = 'uid___A002_X552607_Xa39.ms.wvr',
    caltable = 'uid___A002_X552607_Xa39.ms.wvr.smooth',
    smoothtype = 'mean',
    smoothtime = 6.048)
  
  
  # au.plotWVRSolutions(caltable='uid___A002_X552607_Xa39.ms.wvr.smooth', spw='17', antenna='DV04',
  #   yrange=[-180,180],subplot=22, interactive=False,
  #   figfile='uid___A002_X552607_Xa39.ms.wvr.smooth.plots/uid___A002_X552607_Xa39.ms.wvr.smooth') 
  
  #Note: If you see wraps in these plots, try changing yrange or unwrap=True 
  #Note: If all plots look strange, it may be a bad WVR on the reference antenna.
  #      To check, you can set antenna='' to show all baselines.
  

# Generation of the Tsys cal table
mystep = 3
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_X552607_Xa39.ms.tsys') 
  gencal(vis = 'uid___A002_X552607_Xa39.ms',
    caltable = 'uid___A002_X552607_Xa39.ms.tsys',
    caltype = 'tsys')
  
  # au.plotbandpass(caltable='uid___A002_X552607_Xa39.ms.tsys', overlay='time', 
  #   xaxis='freq', yaxis='amp', subplot=22, buildpdf=False, interactive=False,
  #   showatm=True,pwv='auto',chanrange='5~123',showfdm=True, 
  #   field='', figfile='uid___A002_X552607_Xa39.ms.tsys.plots.overlayTime/uid___A002_X552607_Xa39.ms.tsys') 
  
  
  # es.checkCalTable('uid___A002_X552607_Xa39.ms.tsys', msName='uid___A002_X552607_Xa39.ms', interactive=False) 
  

# Application of the WVR and Tsys cal tables
mystep = 4
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  
  
  from recipes.almahelpers import tsysspwmap
  tsysmap = tsysspwmap(vis = 'uid___A002_X552607_Xa39.ms', tsystable = 'uid___A002_X552607_Xa39.ms.tsys')
  
  
  
  applycal(vis = 'uid___A002_X552607_Xa39.ms',
    field = '0',
    spw = '17,19,21,23',
    gaintable = ['uid___A002_X552607_Xa39.ms.tsys', 'uid___A002_X552607_Xa39.ms.wvr.smooth'],
    gainfield = ['0', ''],
    interp = 'linear,linear',
    spwmap = [tsysmap,[]],
    calwt = T,
    flagbackup = F)
  
  
  
  # Note: J1337-129 didn't have any Tsys measurement, and I couldn't find any close measurement. But this is not a science target, so this is probably Ok.
  
  applycal(vis = 'uid___A002_X552607_Xa39.ms',
    field = '2',
    spw = '17,19,21,23',
    gaintable = ['uid___A002_X552607_Xa39.ms.tsys', 'uid___A002_X552607_Xa39.ms.wvr.smooth'],
    gainfield = ['2', ''],
    interp = 'linear,linear',
    spwmap = [tsysmap,[]],
    calwt = T,
    flagbackup = F)
  
  
  
  applycal(vis = 'uid___A002_X552607_Xa39.ms',
    field = '3',
    spw = '17,19,21,23',
    gaintable = ['uid___A002_X552607_Xa39.ms.tsys', 'uid___A002_X552607_Xa39.ms.wvr.smooth'],
    gainfield = ['3', ''],
    interp = 'linear,linear',
    spwmap = [tsysmap,[]],
    calwt = T,
    flagbackup = F)
  
  
  
  applycal(vis = 'uid___A002_X552607_Xa39.ms',
    field = '4',
    spw = '17,19,21,23',
    gaintable = ['uid___A002_X552607_Xa39.ms.tsys', 'uid___A002_X552607_Xa39.ms.wvr.smooth'],
    gainfield = ['4', ''],
    interp = 'linear,linear',
    spwmap = [tsysmap,[]],
    calwt = T,
    flagbackup = F)
  
  # es.getCalWeightStats('uid___A002_X552607_Xa39.ms') 
  

# Split out science SPWs and time average
mystep = 5
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_X552607_Xa39.ms.split') 
  split(vis = 'uid___A002_X552607_Xa39.ms',
    outputvis = 'uid___A002_X552607_Xa39.ms.split',
    datacolumn = 'corrected',
    spw = '17,19,21,23',
    timebin = '6.048s',
    keepflags = T)
  
  

print "# Calibration"

# Listobs, clear pointing table, and save original flags
mystep = 6
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_X552607_Xa39.ms.split.listobs')
  listobs(vis = 'uid___A002_X552607_Xa39.ms.split',
    listfile = 'uid___A002_X552607_Xa39.ms.split.listobs')
  
  tb.open('uid___A002_X552607_Xa39.ms.split/POINTING', nomodify = False)
  a = tb.rownumbers()
  tb.removerows(a)
  tb.close()
  flagmanager(vis = 'uid___A002_X552607_Xa39.ms.split',
    mode = 'save',
    versionname = 'Original')
  
  

# Initial flagging
mystep = 7
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  # Flagging shadowed data
  
  tflagdata(vis = 'uid___A002_X552607_Xa39.ms.split',
    mode = 'shadow',
    flagbackup = F)
  
  # Flag antenna DV22 which has very high Tsys (1e11) for all fields, spw 2 and 3
  tflagdata(vis = 'uid___A002_X552607_Xa39.ms.split',
    mode = 'manual', antenna='DV22', spw='2,3',
    flagbackup = F)
  
  # Flagging atmospheric line(s)
  
  #tflagdata(vis = 'uid___A002_X552607_Xa39.ms.split',
    #mode = 'manual',
    #spw = '0:0~3839',
    #field = '2',
    #flagbackup = F)
  
  # Flagging atmospheric line(s)
  
  #tflagdata(vis = 'uid___A002_X552607_Xa39.ms.split',
    #mode = 'manual',
    #spw = '1:0~3839',
    #field = '2',
    #flagbackup = F)
  
  # Flagging atmospheric line(s)
  
  tflagdata(vis = 'uid___A002_X552607_Xa39.ms.split',
    mode = 'manual',
    spw = '2:2000~3800',
    field = '2',
    flagbackup = F)
  
  # Flagging atmospheric line(s)
  
  tflagdata(vis = 'uid___A002_X552607_Xa39.ms.split',
    mode = 'manual',
    spw = '3:0~3839',
    field = '2',
    flagbackup = F)
  
  

# Putting a model for the flux calibrator(s)
mystep = 8
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  setjy(vis = 'uid___A002_X552607_Xa39.ms.split',
    field = '2', # Titan
    spw = '0,1,2,3',
    standard = 'Butler-JPL-Horizons 2010')
  
  os.system('rm -rf uid___A002_X552607_Xa39.ms.split.setjy.field*.png') 
  for i in ['2']:
    plotms(vis = 'uid___A002_X552607_Xa39.ms.split',
      xaxis = 'uvdist',
      yaxis = 'amp',
      ydatacolumn = 'model',
      field = i,
      spw = '0,1,2,3',
      avgchannel = '9999',
      coloraxis = 'spw',
      plotfile = 'uid___A002_X552607_Xa39.ms.split.setjy.field'+i+'.png')
  

# Save flags before bandpass cal
mystep = 9
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  flagmanager(vis = 'uid___A002_X552607_Xa39.ms.split',
    mode = 'save',
    versionname = 'BeforeBandpassCalibration')
  
  

# Bandpass calibration
mystep = 10
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_X552607_Xa39.ms.split.ap_pre_bandpass') 
  
  gaincal(vis = 'uid___A002_X552607_Xa39.ms.split',
    caltable = 'uid___A002_X552607_Xa39.ms.split.ap_pre_bandpass',
    field = '0', # 3c279
    spw = '0:1536~2304,1:1536~2304,2:1536~2304,3:1536~2304',
    solint = 'int',
    refant = 'DV04',
    calmode = 'ap')
  
  # es.checkCalTable('uid___A002_X552607_Xa39.ms.split.ap_pre_bandpass', msName='uid___A002_X552607_Xa39.ms.split', interactive=False) 
  
  os.system('rm -rf uid___A002_X552607_Xa39.ms.split.bandpass') 
  bandpass(vis = 'uid___A002_X552607_Xa39.ms.split',
    caltable = 'uid___A002_X552607_Xa39.ms.split.bandpass',
    field = '0', # 3c279
    solint = 'inf',
    combine = 'scan',
    refant = 'DV04',
    solnorm = T,
    bandtype = 'B',
    gaintable = 'uid___A002_X552607_Xa39.ms.split.ap_pre_bandpass')
  
  os.system('rm -rf uid___A002_X552607_Xa39.ms.split.bandpass_smooth20ch') 
  
  bandpass(vis = 'uid___A002_X552607_Xa39.ms.split',
    caltable = 'uid___A002_X552607_Xa39.ms.split.bandpass_smooth20ch',
    field = '0', # 3c279
    solint = 'inf,20ch',
    combine = 'scan',
    refant = 'DV04',
    solnorm = T,
    bandtype = 'B',
    gaintable = 'uid___A002_X552607_Xa39.ms.split.ap_pre_bandpass')
  
  # es.checkCalTable('uid___A002_X552607_Xa39.ms.split.bandpass_smooth20ch', msName='uid___A002_X552607_Xa39.ms.split', interactive=False) 
  
  # es.checkCalTable('uid___A002_X552607_Xa39.ms.split.bandpass', msName='uid___A002_X552607_Xa39.ms.split', interactive=False) 
  

# Save flags before gain cal
mystep = 11
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  flagmanager(vis = 'uid___A002_X552607_Xa39.ms.split',
    mode = 'save',
    versionname = 'BeforeGainCalibration')
  
  

# Gain calibration
mystep = 12
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  # Note: the Solar system object used for flux calibration is highly resolved on some baselines.
  # Note: we will first determine the flux of the phase calibrator(s) on a subset of antennas.
  
  clearcal('uid___A002_X552607_Xa39.ms.split',field='J1147-6753s4')
  
  os.system('rm -rf uid___A002_X552607_Xa39.ms.split.phase_short_int') 
  gaincal(vis = 'uid___A002_X552607_Xa39.ms.split',
    caltable = 'uid___A002_X552607_Xa39.ms.split.phase_short_int',
    field = '2', # Titan
    selectdata = T,
    antenna = 'DA41,DA42,DA44,DA45,DA46,DA48,DA49,DA50,DV01,DV04,DV06,DV07,DV08,DV16,DV18,DV19,DV20',
    solint = 'int',
    refant = 'DV04',
    gaintype = 'G',
    calmode = 'p',
    gaintable = 'uid___A002_X552607_Xa39.ms.split.bandpass_smooth20ch')
  
  plotcal(caltable='uid___A002_X552607_Xa39.ms.split.phase_short_int',
            xaxis='time',
            yaxis='phase',
            iteration='spw,antenna',
            subplot=421,
        plotrange=[0,0,-180,180])

 
  gaincal(vis = 'uid___A002_X552607_Xa39.ms.split',
    caltable = 'uid___A002_X552607_Xa39.ms.split.phase_short_int',
    field = '0,3', # 3c279,J1147-6753s4
    selectdata = T,
    solint = 'int',
    refant = 'DV04',
    gaintype = 'G',
    calmode = 'p',
    append = T,
    gaintable = 'uid___A002_X552607_Xa39.ms.split.bandpass_smooth20ch')
  #append=T does not yet verify spw consistency....

  # es.checkCalTable('uid___A002_X552607_Xa39.ms.split.phase_short_int', msName='uid___A002_X552607_Xa39.ms.split', interactive=False) 
  
  os.system('rm -rf uid___A002_X552607_Xa39.ms.split.ampli_short_inf') 
  gaincal(vis = 'uid___A002_X552607_Xa39.ms.split',
    caltable = 'uid___A002_X552607_Xa39.ms.split.ampli_short_inf',
    field = '0,2,3', # 3c279,Titan,J1147-6753s4
    selectdata = T,
    solint = 'inf',
    refant = 'DV04',
    gaintype = 'T',
    calmode = 'ap',
    gaintable = ['uid___A002_X552607_Xa39.ms.split.bandpass_smooth20ch', 'uid___A002_X552607_Xa39.ms.split.phase_short_int'])
  
  # es.checkCalTable('uid___A002_X552607_Xa39.ms.split.ampli_short_inf', msName='uid___A002_X552607_Xa39.ms.split', interactive=False) 
  
  os.system('rm -rf uid___A002_X552607_Xa39.ms.split.flux_short_inf') 
  os.system('rm -rf uid___A002_X552607_Xa39.ms.split.fluxscale') 
  mylogfile = casalog.logfile()
  casalog.setlogfile('uid___A002_X552607_Xa39.ms.split.fluxscale')
  
  fluxscale(vis = 'uid___A002_X552607_Xa39.ms.split', caltable = 'uid___A002_X552607_Xa39.ms.split.ampli_short_inf', fluxtable = 'uid___A002_X552607_Xa39.ms.split.flux_short_inf', reference = '2',refspwmap=[0,1,2,2]) 
  # Titan, the spw 3 is refered to the spw 2 because of a very wide emission line there
  
  casalog.setlogfile(mylogfile)
  
  f = open('uid___A002_X552607_Xa39.ms.split.fluxscale')
  fc = f.readlines()
  f.close()
  
  for phaseCalName in ['J1147-6753s4']:
    for i in range(len(fc)):
      if fc[i].find('Flux density for '+phaseCalName) != -1 and re.search('in SpW=[0-9]+(?: \(ref SpW=[0-9]+\))? is: [0-9]+\.[0-9]+', fc[i]) != None:
        line = (re.search('in SpW=[0-9]+(?: \(ref SpW=[0-9]+\))? is: [0-9]+\.[0-9]+', fc[i])).group(0)
        spwId = (line.split('='))[1].split()[0]
        flux = float((line.split(':'))[1].split()[0])
	print flux
        setjy(vis = 'uid___A002_X552607_Xa39.ms.split',
          field = phaseCalName.replace(';','*;').split(';')[0],
          spw = spwId,
          fluxdensity = [flux,0,0,0])
  
  os.system('rm -rf uid___A002_X552607_Xa39.ms.split.phase_int') 
  gaincal(vis = 'uid___A002_X552607_Xa39.ms.split',
    caltable = 'uid___A002_X552607_Xa39.ms.split.phase_int',
    field = '0,2,3', # 3c279,Titan,J1147-6753s4
    solint = 'int',
    refant = 'DV04',
    gaintype = 'G',
    calmode = 'p',
    gaintable = 'uid___A002_X552607_Xa39.ms.split.bandpass_smooth20ch')
  
  # es.checkCalTable('uid___A002_X552607_Xa39.ms.split.phase_int', msName='uid___A002_X552607_Xa39.ms.split', interactive=False) 
  
  os.system('rm -rf uid___A002_X552607_Xa39.ms.split.flux_inf') 
  gaincal(vis = 'uid___A002_X552607_Xa39.ms.split',
    caltable = 'uid___A002_X552607_Xa39.ms.split.flux_inf',
    field = '0,2,3', # 3c279,Titan,J1147-6753s4
    solint = 'inf',
    refant = 'DV04',
    gaintype = 'T',
    calmode = 'ap',
    gaintable = ['uid___A002_X552607_Xa39.ms.split.bandpass_smooth20ch', 'uid___A002_X552607_Xa39.ms.split.phase_int'])
  
  # es.checkCalTable('uid___A002_X552607_Xa39.ms.split.flux_inf', msName='uid___A002_X552607_Xa39.ms.split', interactive=False) 
  
  os.system('rm -rf uid___A002_X552607_Xa39.ms.split.phase_inf') 
  gaincal(vis = 'uid___A002_X552607_Xa39.ms.split',
    caltable = 'uid___A002_X552607_Xa39.ms.split.phase_inf',
    field = '0,2,3', # 3c279,Titan,J1147-6753s4
    solint = 'inf',
    refant = 'DV04',
    gaintype = 'G',
    calmode = 'p',
    gaintable = 'uid___A002_X552607_Xa39.ms.split.bandpass_smooth20ch')
  
  # es.checkCalTable('uid___A002_X552607_Xa39.ms.split.phase_inf', msName='uid___A002_X552607_Xa39.ms.split', interactive=False) 
  

# Save flags before applycal cal
mystep = 13
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  flagmanager(vis = 'uid___A002_X552607_Xa39.ms.split',
    mode = 'save',
    versionname = 'BeforeApplycal')
  
  

# Application of the bandpass and gain cal tables
mystep = 14
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  for i in ['0']: # 3c279,Titan
    applycal(vis = 'uid___A002_X552607_Xa39.ms.split',
      field = i,
      gaintable = ['uid___A002_X552607_Xa39.ms.split.bandpass_smooth20ch', 'uid___A002_X552607_Xa39.ms.split.phase_int', 'uid___A002_X552607_Xa39.ms.split.flux_inf'],
      gainfield = ['', i, i],
      interp = 'linear,linear',
      calwt = F,
      flagbackup = F)
  
  for i in ['2']: # 3c279,Titan
    applycal(vis = 'uid___A002_X552607_Xa39.ms.split',
      field = i,
      spw='0,1,2',
      gaintable = ['uid___A002_X552607_Xa39.ms.split.bandpass_smooth20ch', 'uid___A002_X552607_Xa39.ms.split.phase_int', 'uid___A002_X552607_Xa39.ms.split.flux_inf'],
      gainfield = ['', i, i],
      interp = 'linear,linear',
      calwt = F,
      flagbackup = F)
  
  applycal(vis = 'uid___A002_X552607_Xa39.ms.split',
    field = '3,4', # HD 100546
    gaintable = ['uid___A002_X552607_Xa39.ms.split.bandpass_smooth20ch', 'uid___A002_X552607_Xa39.ms.split.phase_inf', 'uid___A002_X552607_Xa39.ms.split.flux_inf'],
    gainfield = ['', '3', '3'], # J1147-6753s4
    interp = 'linear,linear',
    calwt = F,
    flagbackup = F)
  

